<?php
require('exceptions.php');
require('input.php');
require('output.php');
require('response.php');
class REST {
	const get=1;
	const post=2;
	const put=3;
	const delete=4;
	const json=1;
	const serialize=2;
	static private $reqUrl='';
	static private $method=self::get;
	static private $datatype=self::json;
	static private $statusCode=200;
	static private $errorString=NULL;
	static private $latest_output_object=NULL;
	static private $latest_input_object=NULL;
	/**
	 * Initialisation method for the class
	 * @throws MethodNotAllowedException
	 * @throws UnsupportedMediaTypeException
	 */
	static function init() {
		self::$reqUrl=preg_replace('$^/api/REST/$', '', $_SERVER['REQUEST_URI']);
		if(strpos($_SERVER['HTTP_ACCEPT'], 'application/json')!==false ) { //Accept: application/json
			self::$datatype=self::json;
			header('Content-Type: application/json');
		}
		elseif(strpos($_SERVER['HTTP_ACCEPT'], 'application/x-php-serialized')!==false) { // Accept: application/x-php-serialized
			self::$datatype=self::serialize;
			header('Content-Type: application/x-php-serialized');
		}
		elseif(strpos(self::$reqUrl, '.json')!==false){ //Request has .json extention
			self::$datatype=self::json;
			header('Content-Type: application/json');
		}
		elseif(strpos(self::$reqUrl, '.ser')!==false){ //Request has .ser extention
			self::$datatype=self::serialize;
			header('Content-Type: application/x-php-serialized');
		} 
		else {
			header('Content-Type: application/json');
			throw new UnsupportedMediaTypeException('Please provide an Accept header of \'application/json\' or \'application/x-php-serialized\' or add a .json or .ser extention');
		}
		self::$reqUrl=preg_replace('/\\.(json|ser)/i', '', self::$reqUrl);
		if(strpos(self::$reqUrl, '?')===false) self::$reqUrl.='?';
		$methodstr=strtolower($_SERVER['REQUEST_METHOD']);
		switch ($methodstr) {
			case 'delete':
				self::$method=self::delete;
			break;
			case 'put':
				self::$method=self::put;
			break;
			case 'post':
				self::$method=self::post;
			break;
			case 'get':
				;
			break;
			default:
				throw new MethodNotAllowedException($methodstr.' is not a valid method');
			break;
		}
	}
	/**
	 * Gets the input given to the API
	 * @return RESTinput
	 */
	static function input() {
		if(self::$latest_input_object===NULL)
			self::$latest_input_object=new RESTinput(self::$reqUrl,self::$method);
		return self::$latest_input_object;
	}
	/**
	 * Gives an object to write output to
	 * @return RESToutput
	 */
	static function output() {
		if(self::$latest_output_object===NULL)
			self::$latest_output_object=new RESToutput;
		return self::$latest_output_object;
	}
	/**
	 * Gives a response object
	 * @param RESToutput $output
	 * @return RESTresponse
	 */
	static function response(RESToutput $output=null) {
		if(self::$statusCode>399) { //Error condition
			$output=self::output()->setData(array('status'=>self::$statusCode,'errorString'=>self::$errorString));
		}
		if($output===null) { //No parameter passed
			$output=self::$latest_output_object;
		}
		if($output===null) { //No latest output object
			throw new NotImplementedException('No data was send by the module');
		}
		return new RESTresponse(self::$datatype,$output,self::$statusCode);
	}
	/**
	 * Prepares an error for sending
	 * @param int $statusCode HTTP status code
	 * @param string $errorString String explaining the error
	 */
	static function error($statusCode,$errorString) {
		self::$statusCode=$statusCode;
		self::$errorString=$errorString;
	}
	/**
	 * Prepares a success message for sending
	 * @param int $statusCode HTTP status code
	 */
	static function success($statusCode) {
		self::$statusCode=$statusCode;
	}
	/**
	 * Convert a numeric array to a numeric array containing arrays with a string key and the value
	 * @param array $array Numeric array
	 * @param string $name Name of the key to create
	 * @return array
	 */
	static function namedArray($array,$name) {
		$retarr=array();
		foreach($array as $value) {
			$new=array($name=>$value);
			$retarr[]=$new;
		}
		return $retarr;
	}
	static function getMethod() {
		return self::$method;
	}
	static function getDataType() {
		return self::$datatype;
	}
}
REST::init();